ELLIPSE = "ellipse"
OVAL = "oval"
BOX = "box"
RECT = "rect"
RECTANGLE = "rectangle"
HEXAGON = "hexagon"
POLYGON = "polygon"
CIRCLE = "circle"
EGG = "egg"
TRIANGLE = "triangle"
PLAIN = "plain"
DIAMOND = "diamond"
TRAPEZIUM = "trapezium"
PARALLELOGRAM = "parallelogram"
HOUSE = "house"
PENTAGON = "pentagon"
OCTAGON = "octagon"
DOUBLE_CIRCLE = "doublecircle"
DOUBLE_OCTAGON = "doubleoctagon"
INV_TRIANGLE = "invtriangle"
INV_TRAPEZIUM = "invtrapezium"
INV_HOUSE = "invhouse"
SQUARE = "square"
STAR = "star"
UNDERLINE = "underline"
CYLINDER = "cylinder"
NOTE = "note"
TAB = "tab"
FOLDER = "folder"
BOX_3D = "box3d"
COMPONENT = "component"
PROMOTER = "promoter"
RPROMOTER = "rpromoter"
LPROMOTER = "lpromoter"
CDS = "cds"
RARROW = "rarrow"
LARROW = "larrow"
