DECLARATION = '<?xml version="1.0"?>'
BACK = "back"
NO_FILL = "o"

SOLID = "solid"
DASHED = "dashed"

MIDDLE = "middle"

BLOCK = "block"
DIAMOND = "diamond"

CENTER = "center"
START = "start"

DEFAULT_TEXT_SIZE = 10
DEFAULT_FONT_FAMILY = "Helvetica"
DEFAULT_FONT_COLOR = "#000000"

DEFAULT_STROKE = "#000000"
DEFAULT_FILL = "#ffffff"

ARRAY = "Array"
CELL = "mxCell"
USER_OBJECT = "UserObject"
GEO = "mxGeometry"
GRAPH = "mxGraphModel"
POINT = "mxPoint"
ROOT = "root"
DIAGRAM = "diagram"

CURVE_INTERVALS = [0.25, 0.5, 0.75]

NONE = "none"
